@@ -6,6 +6,7 @@ import json
 import logging
 import random
 import re
+import signal
 import time as mod_time
 import uuid
 
@@ -31,6 +32,20 @@ KEY_PREFIX = 'r:'  # Prefix of redis-extensions used key
 WARNING_LOG = '``{0}`` used, may be very very very slow when keys\' amount very large'  # ``r.keys()`` and ``r.scan_iter()`` not support use
 
 
+POLL_QUEUE_CONTINUE_FLAG = True
+
+
+# Signal Handler
+def sigintHandler(signum, frame):
+    global POLL_QUEUE_CONTINUE_FLAG
+    POLL_QUEUE_CONTINUE_FLAG = False
+
+
+# signal.SIGKILL, `KILL -9`, unblockable
+for signum in [signal.SIGHUP, signal.SIGINT, signal.SIGTERM, signal.SIGTSTP]:
+    signal.signal(signum, sigintHandler)
+
+
 class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
     """
     Extension of [redis-py](https://github.com/andymccurdy/redis-py)'s StrictRedis.
@@ -1043,7 +1058,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
             logger.error(e)
             return None
 
-    def release_poll_queue_lock(self, delayed):
+    def release_poll_queue_lock(self, delayed, final_logger=None):
         if not delayed:
             return
         item = self.zrange(delayed, 0, 0, withscores=True)
@@ -1051,10 +1066,10 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
             return
         item = item[0][0]
         identifier, queue, name, args = json.loads(item)
-        logger.info('  * Release lock: {0}'.format(identifier))
+        final_logger.info('  * Release lock: {0}'.format(identifier))
         return self.delete_lock(identifier)
 
-    def poll_queue(self, callbacks={}, delayed=KEY_PREFIX + 'delayed:default', unlocked_warning_func=None, enable_auto_zrem=False, enable_queue=False, release_lock_when_launch=True, release_lock_key=None, release_lock_key_expire=1800, release_lock_when_error=True):
+    def poll_queue(self, callbacks={}, delayed=KEY_PREFIX + 'delayed:default', enable_auto_zrem=False, enable_queue=False, process_lock_key=None, release_lock_when_launch=True, release_lock_key=None, release_lock_key_expire=1800, release_lock_when_error=True, delayed_logger=None, unlocked_warning_func=None):
         """
         Consumer of delay execute.
 
@@ -1064,6 +1079,8 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
         ``enable_queue`` indicates whether enable queue or not.
 
+        ``process_lock_key`` indicates acquire process lock key.
+
         ``release_lock_when_launch`` indicates whether release lock when launch or not. This is for ``restart``.
 
         ``release_lock_key`` indicates acquire lock key when ``release_lock_when_launch``.
@@ -1075,31 +1092,48 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
         callbacks = {k: self.__callable_func(v) for k, v in iteritems(callbacks)}
         callbacks = {k: v for k, v in iteritems(callbacks) if v}
 
-        logger.info('>>> Available callbacks ({0}):'.format(len(callbacks)))
+        final_release_lock_key = release_lock_key or delayed
+        final_process_lock_key = process_lock_key or delayed
+        final_logger = delayed_logger or logger
+
+        final_logger.info('>>> Available callbacks ({0}):'.format(len(callbacks)))
         for k, v in iteritems(callbacks):
-            logger.info('  * {0}: {1}'.format(k, v))
+            final_logger.info('  * {0}: {1}'.format(k, v))
 
         if release_lock_when_launch:
-            locked = self.acquire_lock(release_lock_key or delayed, time=release_lock_key_expire)
-            if locked:
-                logger.info('>>> Release pool queue lock start')
-                self.release_poll_queue_lock(delayed)
-                logger.info('>>> Release pool queue lock end')
+            release_lock = self.acquire_lock(final_release_lock_key, time=release_lock_key_expire)
+            if release_lock:
+                final_logger.info('>>> Release process lock start')
+                self.delete_lock(process_lock_key or delayed)
+                final_logger.info('>>> Release process lock end')
+                final_logger.info('>>> Release item lock start')
+                self.release_poll_queue_lock(delayed, final_logger=final_logger)
+                final_logger.info('>>> Release item lock end')
+
+        process_lock = None
+        while POLL_QUEUE_CONTINUE_FLAG:
+            # Release process lock
+            if process_lock:
+                self.release_lock(final_process_lock_key, process_lock)
+
+            # Acquire process lock
+            process_lock = self.acquire_lock(final_process_lock_key)
+            if not process_lock:
+                continue
 
-        while True:
             item = self.zrange(delayed, 0, 0, withscores=True)
 
             if not item or item[0][1] > mod_time.time():
                 mod_time.sleep(.01)
                 continue
 
-            logger.info(item)
+            final_logger.info(item)
 
             item = item[0][0]
             identifier, queue, name, args = json.loads(item)
 
-            locked = self.acquire_lock(identifier)
-            if not locked:
+            item_lock = self.acquire_lock(identifier)
+            if not item_lock:
                 # Call ``unlocked_warning_func`` if exists when ``acquire_lock`` fail
                 if unlocked_warning_func:
                     unlocked_warning_func(queue, name, args)
@@ -1114,16 +1148,16 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
                 try:
                     callbacks[queue](name, args)
                 except Exception as e:
-                    logger.error(e)
+                    final_logger.error(e)
                     if not release_lock_when_error:
                         continue
-                    self.release_lock(identifier, locked)
+                    self.release_lock(identifier, item_lock)
 
             # At least once 最少消费一次
             if not enable_auto_zrem and self.zrem(delayed, item) and enable_queue:
                 self.rpush(self.__queue_key(queue), item)
 
-            self.release_lock(identifier, locked)
+            self.release_lock(identifier, item_lock)
 
     # HotKey Section
     def hotkey(self, gfunc=None, gargs=None, gkwargs=None, sfunc=None, sargs=None, skwargs=None, update_timeout=1000, short_uuid=False):
